#include "mainwindow.h"
#include "ui_mybookreader.h"

#include <QFile>
#include <QKeyEvent>
#include <QTextStream>
#include <QDir>

#ifdef Q_OS_UNIX
#include <unistd.h>
#include <csignal>
#include <sys/socket.h>
#endif

const QString ARG_SET_NAME = "-set";
const QString ARG_LST_NAME = "-list";
const QString LST_FILENAME_DEFAULT = "readlist";

#ifdef Q_OS_UNIX
int MainWindow::sigtermFd[2];
#endif

MainWindow::MainWindow(const QString &args, QWidget *parent) :
        QMainWindow(parent, Qt::Window | Qt::FramelessWindowHint), ui(new Ui::MyBookReader)
{

#ifdef Q_OS_UNIX
    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sigtermFd))
        qFatal("Couldn't create TERM socketpair");
    snTerm = new QSocketNotifier(sigtermFd[1], QSocketNotifier::Read, this);
    connect(snTerm, SIGNAL(activated(int)), this, SLOT(handleSigTerm()));
#endif

    ui->setupUi(this);        

    pX = 0;
    pY = 0;

    QPalette pal;
    pal.setBrush(this->backgroundRole(), QBrush(QPixmap(":/reader_back_s.png")));
    this->setPalette(pal);   

    process = new QProcess(this);

    //.oO0
    ui->label_A->hide();
    ui->books->hide();
    //0Oo.

    m_sets[ARG_SET_NAME] = "";
    m_sets[ARG_LST_NAME] = "";

#ifdef Q_OS_WIN32
    m_apps["default"] = "C:\\Program Files\\Notepad++\\notepad++.exe";
    m_apps["pdf"] = "C:\\Program Files\\Foxit Software\\Foxit Reader\\Foxit Reader.exe";
    m_apps["fb2"] = "C:\\Program Files\\AlReader2\\AlReader2.exe";
    m_apps["djv"] = "C:\\Program Files\\Djvu\\WinDjVew-0.5.exe";
    m_apps["djvu"] = "C:\\Program Files\\Djvu\\WinDjVew-0.5.exe";
#else
    m_apps["default"] = "bluefish";
    m_apps["pdf"] = "xpdf";
    m_apps["fb2"] = "FBReader";
    m_apps["djv"] = "djview";
    m_apps["djvu"] = "djview";
#endif

    foreach(QString key, m_sets.keys()){
        QRegExp exp(key + "\\s(?!-)\\S+");
        int pos = exp.indexIn(args);
        if (pos > -1) {
            m_sets[key] = exp.cap().split(" ").at(1);
        }
    }

    if (!m_sets[ARG_SET_NAME].isEmpty()){
        QFile appListFile(m_sets.value(ARG_SET_NAME));
        QTextStream st(&appListFile);
        if (appListFile.open(QIODevice::ReadOnly | QIODevice::Text)){
            QString k,v;
            QStringList q;
            while (!st.atEnd()){
                q = st.readLine().split("=");
                if (q.size() == 2){
                    k = q.at(0); k = k.trimmed().toLower();
                    v = q.at(1); v = v.trimmed();
                    m_apps[k] = v;
                }
            }
            appListFile.close();
        }
    }

    if (!m_sets[ARG_LST_NAME].isEmpty()){
        m_bookListFile.setFileName(m_sets.value(ARG_LST_NAME));
    } else {
        m_bookListFile.setFileName(LST_FILENAME_DEFAULT);
    }

    m_bookList.clear();

    QTextStream st(&m_bookListFile);
    if (m_bookListFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        while (!st.atEnd()){
            m_bookList.append(st.readLine().trimmed());
         }
         m_bookListFile.close();
    }

    connect(ui->quitButton,SIGNAL(clicked()),this,SLOT(pressQuitButton()));

    //.oO0
    if (!m_bookList.isEmpty()){
        ui->label->hide();
        ui->spaceFrame->hide();
        ui->label_A->show();
        ui->books->show();
    } else {
        ui->openButton->setDisabled(true);
        ui->closeButton->setDisabled(true);
        return;
    }
    //0Oo.

    int r;
    r = 0;

    QTableWidgetItem* item;

    ui->books->setColumnWidth(NAME, 200);
    ui->books->setColumnWidth(AUTHOR, 100);
    ui->books->setColumnWidth(DATEOFCREATION, 40);

    QStringList subContent;

    foreach(QString tmpFilename, m_bookList){

        QString name;
        QString author;
        QString date;

        subContent.clear();

        QString a;
        a = tmpFilename.section(".",0,-2) + ".sub";

        QFile subFile(a);
        QTextStream st(&subFile);

        st.setCodec("UTF-8");
        st.setGenerateByteOrderMark(false);

        if (subFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
            while (!st.atEnd()){
                QString line;
                line = st.readLine().trimmed();
                if (!line.isEmpty()){
                    subContent.append(line);
                }
            }
            subFile.close();
            if (subContent.count() == 3) {
                name = subContent.at(0);
                author = subContent.at(1);
                date = subContent.at(2);
            }
        }

        ui->books->setRowCount(r+1);
        item = new QTableWidgetItem(name);
        item->setToolTip(item->text());
        ui->books->setItem(r,NAME,item);

        item = new QTableWidgetItem(author);
        item->setToolTip(item->text());
        ui->books->setItem(r,AUTHOR,item);

        item = new QTableWidgetItem(date);
        item->setToolTip(item->text());
        item->setTextAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
        ui->books->setItem(r,DATEOFCREATION,item);

        item = new QTableWidgetItem(QDir::toNativeSeparators(tmpFilename));
        item->setToolTip(item->text());
        ui->books->setItem(r,FILENAME,item);

        r++;
    }

    ui->books->sortItems(NAME);

    ui->books->resizeRowsToContents();
    ui->books->setCurrentCell(0,NAME);
    ui->books->setFocus();

    ui->books->installEventFilter(this);

    connect(ui->openButton,SIGNAL(clicked()),this,SLOT(pressOpenButton()));
    connect(ui->closeButton,SIGNAL(clicked()),this,SLOT(pressCloseButton()));
    connect(process,SIGNAL(finished(int)),this,SLOT(pressCloseButton()));

    if (m_bookList.count() == 1) {
        pressOpenButton();
    }

    timerChkExitId = startTimer(200);
}

MainWindow::~MainWindow()
{
    if (process->state() == QProcess::Running){
        process->close();
        process->waitForFinished(-1);
    }

    delete ui;
}

void MainWindow::closeEvent(QCloseEvent *e)
{
    quitReader();
}

void MainWindow::mousePressEvent(QMouseEvent* e)
{
    this->setWindowOpacity(0.5);
    pX = e->x();
    pY = e->y();
    QWidget::mousePressEvent(e);
}

void MainWindow::mouseReleaseEvent(QMouseEvent* e)
{
    this->setWindowOpacity(1.0);
    QWidget::mouseReleaseEvent(e);
}

void MainWindow::mouseMoveEvent(QMouseEvent* e)
{
    this->move(this->x() + e->x() - pX, this->y() + e->y() - pY);
    QWidget::mouseMoveEvent(e);
}

void MainWindow::pressOpenButton()
{
    ui->books->setDisabled(true);
    ui->openButton->setDisabled(true);
    ui->closeButton->setEnabled(true);
    ui->closeButton->setFocus();

    QString name;
    name = ui->books->item(ui->books->currentRow(),FILENAME)->text();

    QString ext;
    ext = name.section(".",-1,-1).toLower();

    if (m_apps.contains(ext)){
        process->start(m_apps.value(ext),QStringList(name));
    } else {
        process->start(m_apps.value("default"),QStringList(name));
    } 
}

void MainWindow::pressCloseButton()
{
    ui->closeButton->setDisabled(true);

    if (process->state() == QProcess::Running){
        process->close();
        process->waitForFinished(-1);
    }

    ui->openButton->setEnabled(true);
    ui->books->setEnabled(true);
    ui->books->setFocus();
}

void MainWindow::pressQuitButton()
{
    quitReader();
}

void MainWindow::quitReader()
{
    if (process->state() == QProcess::Running){
        process->close();
        process->waitForFinished(-1);
    }
    this->close();
}

bool MainWindow::eventFilter(QObject *target, QEvent *event)
{
    bool result;
    result = false;

    if (target == ui->books){
        if (event->type() == QEvent::KeyPress) {
            QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);

            int cp;
            cp = ui->books->currentRow();

            switch (keyEvent->key()){
            case Qt::Key_Up:
                if (cp == 0){
                    result = true;
                }
                break;
            case Qt::Key_Down:
                if (cp == ui->books->rowCount() - 1){
                    result = true;
                }
                break;
            case Qt::Key_Enter:
            case Qt::Key_Return:
            case Qt::Key_Space:
                pressOpenButton();
                result = true;
                break;
            default:;
            }
        }
    }
    return result;
}

void MainWindow::timerEvent(QTimerEvent* event)
{
    int id;
    id = event->timerId();
    if (id == timerChkExitId){
        if (!m_bookListFile.exists()){
            quitReader();
        }
        if ((m_bookList.count() == 1) && (process->state() != QProcess::Running)){
            quitReader();
        }
    } else {
        QWidget::timerEvent(event);
    }
}

#ifdef Q_OS_UNIX
void MainWindow::termSignalHandler(int unused)
{
    char a = 1;
    ::write(sigtermFd[0], &a, sizeof(a));
}
#endif


void MainWindow::handleSigTerm()
{
#ifdef Q_OS_UNIX
    snTerm->setEnabled(false);
    char tmp;
    ::read(sigtermFd[1], &tmp, sizeof(tmp));

    quitReader();

    snTerm->setEnabled(true);
#endif
}

