#ifndef ANYBOX_H
#define ANYBOX_H

#include "myfolder.h"
#include "mysourcefile.h"
#include "category.h"
#include "myabcfile.h"

#include <QSet>
#include <QDate>
#include <QTime>
#include <QStringList>
#include <QFile>
#include <QProcess>
#include <QPixmap>

/*
  AnyBox (Коробка)
  Основан на MyFolder
  При создании, на основе расположения каталогов на диске, формируется древовидная структура
   <AnyBox>
      |--<MyAbcFolder> [a-z][a-z]
      |    |--<MyAbcFolder> [a-z][a-z]
      |    |    |
      |    |    |--<MyAbcFile> abc
      |    |
      |    |--<MyAbcFolder> [a-z][a-z]
      |      ...
      |    |--<MyAbcFolder> [a-z][a-z]
      |
      |--<MyAbcFolder> [a-z][a-z]
      |  ...
      |--<MyAbcFolder> [a-z][a-z]

  Содержит настройки:
    m_ext         - список расширений файлов, хранимых в коробке
    m_extPlayList - расширение файла, содержащего список воспроизведения
    m_execPreview - приложение для предварительной демонстрации списка воспроизведения
    m_execPresent - приложение для окончательной демонстрации списка воспроизведения

    m_category    - описание категории
    m_dateFormat  - формат даты, принятой для коробки

  AnyBox содержит список ABC-файлов, в которых есть помеченные записи. Эти сведения заносятся
  в m_subsetsOfMarkedAbcFiles при помощи методов markAbcFile(...) и unmarkAbcFile(...), которые
  вызываются по сигналам от этих ABC-файлов. Списков может быть несколько. Их можно объединять
  методом uniteSubsets(). Получить списки помеченных записей можно методом markedEntries().
  Также  возможно запоминать (memorizeMarked()) и восстанавливать (restoreMarked()) помеченные записи.

  Метод storeMarkedEntries() позволяет сохранить помеченные Abc-файлы на диск.

  Метод makePlayListFile(...) позволяет создать файл на диске (список воспроизведения), который содержит
  список абсолютных имен файлов (на одну строку - один файл).

  Методы previewSelected(...) и presentSelected(...) запускают процесс демонстрации помеченных записей

*/

class AnyBox : public MyFolder
{
    Q_OBJECT

    quint32 m_count;

    QSet<QString> m_ext;
    QString m_extPlayList;
    QString m_execPreview;
    QString m_execPresent;

    QPixmap m_boxPixmap;
    QString m_welcomeLabel;

    typedef QSet<MyAbcFile*> subsetOfMarkedAbcFiles;
    QList<subsetOfMarkedAbcFiles> m_subsetsOfMarkedAbcFiles;
    QList<subsetOfMarkedAbcFiles> m_subsetsOfMarkedAbcFilesMemory;

protected:
    QString m_dateFormat;
    bool m_useSubFile;

    Category m_category;
    QString m_myCategories; // Имена и описание части F категории в виде строки

private slots:
    void markAbcFile(MyAbcFile* abcFile, int subset);
    void unmarkAbcFile(MyAbcFile* abcFile, int subset);

public:
    explicit AnyBox(const QString &name, MyUnit* parent);

    void setExt(const QString &);
    void setExtPlayList(const QString &);
    void setExecPreview(const QString &);
    void setExecPresent(const QString &);
    void setDateFormat(const QString &);  

    quint32 getCount();
    void addCount(quint32);

    QSet<QString> ext();
    QString extPlayList();
    QString execPreview();
    QString execPresent();
    QString dateFormat();

    QString makePlayListFile(const QString &, QList<Entry*>);
    QString makePlayListFile(const QString &);    

    QString previewSelected(const QString &, QProcess*);
    QString previewSelected(int, const QString &, QProcess*);
    QString presentSelected(const QString &, QProcess*);
    QString previewSelected(QProcess*);
    QString previewSelected(int, QProcess*);
    QString presentSelected(QProcess*);

    QList<Entry*>markedEntries();
    QList<Entry*>markedEntries(int);

    void storeMarkedEntries();
    int markedEntriesCount();
    int markedEntriesCount(int subset);

    virtual quint32 startValueCategory(MySourceFile*);

    virtual void makeSubFile(QString);

    Category category();    

    QSet<MyAbcFile*> markedAbcFiles();
    QSet<MyAbcFile*> markedAbcFiles(int subset);

    QSet<MyAbcFile*> markAllEntries(int subset);
    void unmarkAllEntries();

    void deleteSubset(int subset);
    void uniteSubsets();

    void memorizeMarked();
    void restoreMarked();

    void setUseSubFile(bool);
    bool useSubFile();

    void setMyCategories(const QString &);
//    virtual QString myCategories();
    virtual void setCategoryLabels();

    void setBoxPixmap(const QPixmap pixmap);
    void setWelcomeLabel(const QString label);

    QPixmap boxPixmap();
    QString welcomeLabel();

signals:
    void showReduce();

public slots:

};

#endif // BOX_H
