#ifndef MYABCFILE_H
#define MYABCFILE_H

#include "myfile.h"
#include "entry.h"
#include <QMap>
#include <QSet>

/*
  Файл с описаниями
  Основан на MyFile
    |--anybox
        |--aa
        |--ab
        |   |--aa
        |   |   |--abc (MyAbcFile)
        |   |   |--aa.ext
        |   |   |--ab.ext
        |   |  ...
        |   |
        |   |--ab
        | ...
        |--ac
       ...
   Содержит записи в переменной m_entries. Записи поименованы в соответствии с именами файлов (без расширений),
   находящихся в том же каталоге, что и Abc-файл. Записи можно помечать (метод markEntry()) и хранить их в списке
   помеченных записей. Таких списков может быть несколько. Эти списки можно запоминать во временной
   переменной (метод memorizeSubsets()) и восстанавливать (метод restoreSubsetsFromMemory()) при
   необходимости. Списки можно объединять в один (метод uniteSubsets())
   Для упрощения поиска элемента MyAbcFile, ему присваивается имя "abc_rema", где rema - соответствует
   местонахождению файла в структуре anybox
*/

class MyAbcFile : public MyFile
{
    Q_OBJECT

    QString m_rema;                         // местонахождение Abc-файла (для приведенной схемы - abaa)
    QMap<QString,Entry> m_entries;

    typedef QSet<Entry*> subset;            // множество помеченных записей
    QList<subset> m_subsets;                // список множеств помеченных записей
    QList<subset> m_subsetsMemory;          // для временного запоминания списка множеств

public:
    MyAbcFile(MyUnit* parent);

    Entry* getEntry(QString);               // получить запись (в нашем примере, метод getEntry("aa")
                                            // вернет запись, соответствующую файлу aa.ext
    QString getEntryPart(QString,int);      // получение определенного поля в записи
    QStringList getUsedName();              // список имен существующих записей
    QList<Entry> getAll();                  // возвращает список всех записей из Abc-файла
    QString rema();                         // возвращает местонахождение Abc-файла

    bool update(Entry);                     // обновить/добавить запись в m_entries
    bool store();                           // сохранение m_entries в файле на диске

    int countEntries();                     // количество записей в Abc-файле
    bool entryExist(QString);               // наличие записи с заданным именем

    bool markEntry(Entry*, int sub);        // добавить запись в множество sub
    bool unmarkEntry(Entry*, int sub);      // удалить запись из множества sub
    int markAllEntries(int sub);            // добавить все записи в множество sub

    QList<Entry*> markedEntries();          // вернуть список записей из всех множеств
    QList<Entry*> markedEntries(int sub);   // вернуть список записей из множества sub
    QList<Entry*> allEntries();             // вернуть список всех записей этого abc-файла

    void deleteSubsets();                   // очистить все множества помеченных записей
    void deleteSubset(int);                 // удалить множество из списка помеченных записей

    void uniteSubsets();                    // объединить все множества в одно

    int markedEntriesCount();               // количество записей во всех множествах
    int markedEntriesCount(int sub);        // количество записей в множестве sub

    void memorizeSubsets();                 // запомнить множества с помеченными записями
    void restoreSubsetsFromMemory();        // восстановить множества с помеченными записями

signals:
    void markAbcFile(MyAbcFile*, int);
    void unmarkAbcFile(MyAbcFile*, int);

};

#endif // MYABCFILE_H
