#include "mainwindow.h"
#include "ui_mymplayer.h"

#include <QTime>
#include <QKeyEvent>
#include <QTextStream>
#include <QDir>
#include <QFileInfo>
#include <QTextCodec>

#ifdef Q_OS_UNIX
#include <unistd.h>
#include <csignal>
#include <sys/socket.h>
#endif

const QString ARG_MPL_NAME = "-mplayer";
const QString ARG_SET_NAME = "-set";
const QString ARG_LST_NAME = "-list";

#ifdef Q_OS_WIN32
const QString APP_DEFAULT = "c:\\Program Files\\MPlayer\\mplayer.exe";
#else
const QString APP_DEFAULT = "mplayer";
#endif

const QString LST_FILENAME_DEFAULT = "playlist";

#ifdef Q_OS_UNIX
int MainWindow::sigtermFd[2];
#endif

MainWindow::MainWindow(const QString &args, QWidget *parent) :
            QMainWindow(parent,Qt::Window | Qt::FramelessWindowHint), ui(new Ui::MyMPlayer)

{        

#ifdef Q_OS_UNIX
    if (::socketpair(AF_UNIX, SOCK_STREAM, 0, sigtermFd))
        qFatal("Couldn't create TERM socketpair");
    snTerm = new QSocketNotifier(sigtermFd[1], QSocketNotifier::Read, this);
    connect(snTerm, SIGNAL(activated(int)), this, SLOT(handleSigTerm()));
#endif

    ui->setupUi(this);

    codecNames << "Windows-1251" << "KOI8-R" << "UTF-8";

    codecs.clear();

    foreach(QByteArray name, codecNames){
        codecs << QTextCodec::codecForName(name);
    }

    currentCodecId = 0;
    currentCodec = codecs.at(currentCodecId);

    pX = 0;
    pY = 0;

    QPalette pal;
    pal.setBrush(this->backgroundRole(), QBrush(QPixmap(":/back_s.png")));
    this->setPalette(pal);

    this->setWindowOpacity(1.0);

    currentMelody = 0;
    currentVolume = 100;
    stepVolume = 3;

    time  = 0.0;
    timeA = 0.0;
    timeB = 0.0;

    timerVolumeDownId = 0;
    timerTimePosId = 0;
    timerChangeInfoId = 0;
    timerChkExitId = 0;


    infoTitle << "Title" << "Artist" << "Album" << "Year" ;

    unknown = "- - -";

    foreach(QString tmp, infoTitle){
        info[tmp]  = unknown;
    }
    infoCurrent = 0;

    QString a;
    a = infoTitle.at(infoCurrent);
    ui->label_D->setText(a);
    ui->infoMelody->setText(info.value(a));

    m_sets[ARG_MPL_NAME] = "";
    m_sets[ARG_SET_NAME] = "";
    m_sets[ARG_LST_NAME] = "";

    // Разбор параметров командной строки (заполнение m_sets)

    foreach(QString key, m_sets.keys()){
        QRegExp exp(key + "\\s(?!-)\\S+");
        int pos = exp.indexIn(args);
        if (pos > -1) {
            m_sets[key] = exp.cap().split(" ").at(1);
        }
    }

    // Формирование списка воспроизведения

    if (!m_sets[ARG_LST_NAME].isEmpty()){
        m_playlistFile.setFileName(m_sets.value(ARG_LST_NAME));
    } else {
        m_playlistFile.setFileName(LST_FILENAME_DEFAULT);
    }

    m_playlist.clear();

    QTextStream st(&m_playlistFile);
    if (m_playlistFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        while (!st.atEnd()){
            m_playlist.append(st.readLine().trimmed());
         }
         m_playlistFile.close();
    }

    foreach(QString tmp_name, m_playlist){

        QString subTitle, subAuthor, subYear;

        subData(tmp_name, subTitle, subAuthor, subYear);

        m_subTitles << subTitle;
        m_subAuthors << subAuthor;
        m_subYears << subYear;
    }

    // Приложение для воспроизведения

    process = new QProcess(this);

    if (!m_sets[ARG_MPL_NAME].isEmpty()){
        m_appPlay.setFileName(m_sets.value(ARG_MPL_NAME));
    } else {
        m_appPlay.setFileName(APP_DEFAULT);
    }

    // Формирование строки запуска приложения для воспроизведения

    if (!m_sets[ARG_SET_NAME].isEmpty()){
        m_cmd = "\"" + m_appPlay.fileName() + "\"" + " -include " + m_sets.value(ARG_SET_NAME) +
                " -slave " + "\"%1\"";
    } else {
        m_cmd = "\"" + m_appPlay.fileName() + "\"" + " -slave " + "\"%1\"";
    }

    if (!fileExists(m_appPlay.fileName()) || m_playlist.isEmpty()){

        play = false;

        connect(ui->buttonOff, SIGNAL(clicked()), this, SLOT(close()));

        ui->buttonPlay->setDisabled(true);
        ui->buttonHome->setDisabled(true);
        ui->buttonPrevious->setDisabled(true);
        ui->buttonNext->setDisabled(true);
        ui->buttonEnd->setDisabled(true);                

    } else {

        play = true;

        connect(this,SIGNAL(destroyed()),process,SLOT(kill()));

        connect(process,SIGNAL(started()),this,SLOT(reqInfo()));
        connect(process,SIGNAL(finished(int)),this,SLOT(playMelody()));

        connect(process,SIGNAL(readyReadStandardOutput()),this,SLOT(getInfo()));

        connect(ui->buttonOff, SIGNAL(clicked()), this, SLOT(pressOff()));

        connect(ui->buttonNext, SIGNAL(clicked()), this, SLOT(pressNext()));
        connect(ui->buttonNext, SIGNAL(clicked()),this,SLOT(reqInfo()));

        connect(ui->buttonPrevious, SIGNAL(clicked()), this, SLOT(pressPrev()));
        connect(ui->buttonPrevious, SIGNAL(clicked()),this,SLOT(reqInfo()));

        connect(ui->buttonHome, SIGNAL(clicked()), this, SLOT(pressHome()));
        connect(ui->buttonHome, SIGNAL(clicked()),this,SLOT(reqInfo()));

        connect(ui->buttonEnd, SIGNAL(clicked()), this, SLOT(pressEnd()));
        connect(ui->buttonEnd, SIGNAL(clicked()),this,SLOT(reqInfo()));

        connect(ui->buttonPlay, SIGNAL(clicked()), this, SLOT(pressPlay()));

        process->start(m_cmd.arg(m_playlist.at(currentMelody)));

        timerTimePosId = startTimer(1000);
        timerChangeInfoId = startTimer(2000);
        timerChkExitId = startTimer(200);
    }
}

MainWindow::~MainWindow()
{        
    delete ui;    
}

void MainWindow::reqInfo()
{              
    process->write("get_meta_title\n");
    process->write("get_meta_artist\n");
    process->write("get_meta_album\n");
    process->write("get_meta_year\n");
    process->write("get_time_length\n");
    process->write("volume 100 1\n");
}

void MainWindow::getInfo()
{       
    QFileInfo i(m_playlist.at(currentMelody));
    ui->fileName->setText(QDir::toNativeSeparators(i.absolutePath()) + "\n" + i.fileName());

    QString txt;
    QString timeFormat("hh:mm:ss");
    QStringList lst;    
    txt = currentCodec->toUnicode(process->readAllStandardOutput());
    lst = txt.split("\n");
    QString t;
    foreach(QString tmp, lst){
        t = tmp.section('\'',1,1);        
        if (tmp.contains("ANS_META_TITLE")){                        
            if (t.isEmpty()){
                info["Title"] = m_subTitles.at(currentMelody);
            } else {                
                info["Title"] = t;
            }
        }
        if (tmp.contains("ANS_META_ARTIST")){            
            if (t.isEmpty()){
                info["Artist"] = m_subAuthors.at(currentMelody);
            } else {
                info["Artist"] = t;
            }
        }
        if (tmp.contains("ANS_META_ALBUM")){
            if (t.isEmpty()){
                info["Album"] = unknown;
            } else {
                info["Album"] = t;
            }
        }
        if (tmp.contains("ANS_META_YEAR")){
            if (t.isEmpty()){
                info["Year"] = m_subYears.at(currentMelody);
            } else {
                info["Year"] = t;
            }
        }
        if (tmp.contains("ANS_LENGTH")){
            QStringList t;
            t = tmp.split("=");
            time = t.at(1).toFloat();
            QTime a(0,0,0);
            a = a.addSecs(qRound(time));
            ui->totalTime->setText(a.toString(timeFormat));
        }
        if (tmp.contains("ANS_TIME_POSITION")){
            QStringList t;
            t = tmp.split("=");
            timeA = t.at(1).toFloat();            
            timeB = time - timeA;
            QTime a(0,0,0);
            a = a.addSecs(qRound(timeA));
            ui->elapseTime->setText(a.toString(timeFormat));
            QTime b(0,0,0);
            b = b.addSecs(qRound(timeB));
            ui->remainTime->setText(b.toString(timeFormat));
            QTime c(0,0,0);
            c = c.addSecs(qRound(time));
            ui->totalTime->setText(c.toString(timeFormat));
        }
    }
}

void MainWindow::pressNext()
{    
    volumeDownSoft();
    ui->buttonPlay->setIcon(QIcon(":/pl_pause_s.png"));       
}

void MainWindow::pressPrev()
{   
    volumeDownSoft();
    ui->buttonPlay->setIcon(QIcon(":/pl_pause_s.png"));    
    changeCurrentMelody(false);    
    changeCurrentMelody(false);
}

void MainWindow::pressHome()
{
    volumeDownSoft();
    ui->buttonPlay->setIcon(QIcon(":/pl_pause_s.png"));
    currentMelody = m_playlist.count()-1;
}

void MainWindow::pressEnd()
{    
    volumeDownSoft();
    ui->buttonPlay->setIcon(QIcon(":/pl_pause_s.png"));
    currentMelody = m_playlist.count() - 1;
    changeCurrentMelody(false);     
}

void MainWindow::pressPlay()
{    
    process->write("pause\n");
    play = !play;
    if (play){
        ui->buttonPlay->setIcon(QIcon(":/pl_pause_s.png"));
        ui->label_G->setText("Pause");
    } else {
        ui->buttonPlay->setIcon(QIcon(":/pl_play_s.png"));
        ui->label_G->setText("Play");
    }
}

void MainWindow::pressOff()
{    
    disconnect(process,SIGNAL(finished(int)),this,SLOT(playMelody()));
    connect(process,SIGNAL(finished(int)),this,SLOT(close()));
    volumeDownSoft();
}

void MainWindow::volumeDownSoft()
{    
    foreach(QString tmp, infoTitle){
        info[tmp]  = unknown;
    }
    if (play) {
        timerVolumeDownId = startTimer(75);        
        while (currentVolume >= 33){
            QApplication::processEvents();
        }
        killTimer(timerVolumeDownId);
        timerVolumeDownId = 0;
    }
    if (process->state() == QProcess::Running){
        process->write("quit\n");
    }
}

void MainWindow::timerEvent(QTimerEvent* event)
{
    int id;
    id = event->timerId();
    if (id == timerVolumeDownId || id == timerTimePosId ||
        id == timerChangeInfoId || id == timerChkExitId){
        if (id == timerVolumeDownId){            
            if (play) {
                process->write("volume -1\n");
                currentVolume -= stepVolume;
            }
        } else if (id == timerTimePosId){
            if (play) {
                process->write("get_time_pos\n");
            }            
        } else if (id == timerChangeInfoId){
            infoCurrent++;
            if (infoCurrent >= info.count()){
                infoCurrent = 0;
            }
            QString a;
            a = infoTitle.at(infoCurrent);
            ui->label_D->setText(a);
            ui->infoMelody->setText(info.value(a));
        } else if (id == timerChkExitId){
            if (!m_playlistFile.exists()){
                pressOff();
            }
        }
    } else {
        QWidget::timerEvent(event);
    }
}

void MainWindow::playMelody()
{
    play = true;
    currentVolume = 100;
    infoCurrent = -1;
    changeCurrentMelody(true);

    process->start(m_cmd.arg(m_playlist.at(currentMelody)));
}

void MainWindow::changeCurrentMelody(bool direct)
{
    if (direct){
        currentMelody++;
    } else {
        currentMelody--;
    }
    if (currentMelody >= m_playlist.count()){
        currentMelody = 0;
    }
    if (currentMelody < 0){
        currentMelody = m_playlist.count() - 1;
    }
}

void MainWindow::closeEvent(QCloseEvent *e)
{
    pressOff();
}

void MainWindow::keyPressEvent(QKeyEvent* pe)
{
    switch (pe->key()){
    case Qt::Key_Escape:
        if (m_playlist.isEmpty()){
            this->close();
        } else {
            pressOff();
        }
        break;
    case Qt::Key_Greater:
        if (!m_playlist.isEmpty()) {
            pressNext();
        }
        break;
    case Qt::Key_Less:
        if (!m_playlist.isEmpty()) {
            pressPrev();
        }
        break;
    case Qt::Key_Home:
        if (!m_playlist.isEmpty()) {
            pressHome();
        }
        break;
    case Qt::Key_End:
        if (!m_playlist.isEmpty()) {
            pressEnd();
        }
        break;
    case Qt::Key_C:
        if (currentCodecId == codecs.count() - 1){
            currentCodecId = 0;
        } else {
            currentCodecId++;
        }
        ui->infoMelody->setText("Use '" + codecNames.at(currentCodecId) + "' text codec");
        currentCodec = codecs.at(currentCodecId);
        reqInfo();
        break;
    default:
        QWidget::keyPressEvent(pe);
    }
}

void MainWindow::mousePressEvent(QMouseEvent* e)
{
    this->setWindowOpacity(0.5);
    pX = e->x();
    pY = e->y();
    QWidget::mousePressEvent(e);
}

void MainWindow::mouseReleaseEvent(QMouseEvent* e)
{
    this->setWindowOpacity(1.0);
    QWidget::mouseReleaseEvent(e);
}

void MainWindow::mouseMoveEvent(QMouseEvent* e)
{
    this->move(this->x() + e->x() - pX, this->y() + e->y() - pY);
    QWidget::mouseMoveEvent(e);
}

#ifdef Q_OS_UNIX
void MainWindow::termSignalHandler(int unused)
{
    char a = 1;
    ::write(sigtermFd[0], &a, sizeof(a));
}
#endif


void MainWindow::handleSigTerm()
{
#ifdef Q_OS_UNIX
    snTerm->setEnabled(false);
    char tmp;
    ::read(sigtermFd[1], &tmp, sizeof(tmp));

    pressOff();

    snTerm->setEnabled(true);
#endif
}


bool MainWindow::subData(const QString &filename, QString &name, QString &author, QString &date)
{
    bool result = false;

    name = unknown;
    author = unknown;
    date = unknown;

    QStringList subContent;

    QString a;
    a = filename.section(".",0,-2) + ".sub";

    QFile subFile(a);
    QTextStream st(&subFile);

    st.setCodec("UTF-8");
    st.setGenerateByteOrderMark(false);

    if (subFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
        result = true;
        while (!st.atEnd()){
            QString line;
            line = st.readLine().trimmed();
            if (!line.isEmpty()){
                subContent.append(line);
            }
        }
        subFile.close();
        if (subContent.count() == 3) {
            name = subContent.at(0);
            author = subContent.at(1);
            date = subContent.at(2);
        }
    }

    return result;
}

bool MainWindow::fileExists(const QString & name)
{
    QString n = name.trimmed();

    if (QDir::isAbsolutePath(n)){
        return QFile::exists(n);
    }

    bool result = false;

    QChar sep = QDir::separator();

    QStringList paths;
    QString envPath(qgetenv("PATH"));

    paths << "";

    if (sep == '/') {
        paths = envPath.split(":");
    } else {
        envPath.remove(QRegExp("^\\w+="));
        paths = envPath.split(";");
    }

    foreach(QString path, paths){

        path = path.trimmed();

        if (!path.isEmpty() && !path.endsWith(sep)){
            path.append(sep);
        }

        QString f = path.append(n);

        if (QFile::exists(f)){
            result = true;
            break;
        }
    }

    return result;
}
